@@ -34,6 +34,7 @@ typedef struct account
 	char *server;
 	
 	int auto_connect;
+	int auto_reconnect_delay;
 	int reconnect;
 	
 	set_t *set;
@@ -51,9 +52,11 @@ void account_on( irc_t *irc, account_t *a );
 void account_off( irc_t *irc, account_t *a );
 
 char *set_eval_account( set_t *set, char *value );
+char *set_eval_account_reconnect_delay( set_t *set, char *value );
+int account_reconnect_delay( account_t *a );
 
-#define ACC_SET_NOSAVE		1
-#define ACC_SET_OFFLINE_ONLY	2
-#define ACC_SET_ONLINE_ONLY	4
+#define ACC_SET_NOSAVE		0x01
+#define ACC_SET_OFFLINE_ONLY	0x02
+#define ACC_SET_ONLINE_ONLY	0x04
 
 #endif
